
set echo on

DROP TABLE errors;
DROP PACKAGE error;

CREATE TABLE errors
( package_owner  VARCHAR2(30)
, package_name   VARCHAR2(30)
, procedure_name VARCHAR2(30)
, action         VARCHAR2(100)
, business_date  DATE
, business_key   VARCHAR2(1000)       
, error_ts       TIMESTAMP
, error_msg      VARCHAR2(4000)
);

CREATE PACKAGE error AS
   PROCEDURE log( p_owner         IN errors.package_owner%TYPE,
                  p_package       IN errors.package_name%TYPE,
                  p_procedure     IN errors.procedure_name%TYPE,
                  p_action        IN errors.action%TYPE,
                  p_business_date IN errors.business_date%TYPE,
                  p_business_key  IN errors.business_key%TYPE DEFAULT NULL,
                  p_error         IN VARCHAR2 DEFAULT NULL );
END;
/

CREATE PACKAGE BODY error AS

   PROCEDURE log( p_owner         IN errors.package_owner%TYPE,
                  p_package       IN errors.package_name%TYPE,
                  p_procedure     IN errors.procedure_name%TYPE,
                  p_action        IN errors.action%TYPE,
                  p_business_date IN errors.business_date%TYPE,
                  p_business_key  IN errors.business_key%TYPE DEFAULT NULL,
                  p_error         IN VARCHAR2 DEFAULT NULL ) IS

      v_error errors.error_msg%TYPE := NVL(p_error,SQLERRM);

      PRAGMA AUTONOMOUS_TRANSACTION;

   BEGIN

      INSERT INTO errors 
         ( package_owner, package_name, procedure_name, action,
           business_date, business_key, error_ts, error_msg )
      VALUES
         ( p_owner, p_package, p_procedure, p_action,
           p_business_date, p_business_key, SYSTIMESTAMP, v_error );

      COMMIT;

   END log;

END error;
/


