
set echo on
set serveroutput on format wrapped

TRUNCATE TABLE errors;

DECLARE

   v_package        errors.package_name%TYPE   := 'SUBTYPE_ENCAPSULATION';
   v_procedure      errors.procedure_name%TYPE := 'CUSTOMER_LOAD_EG';
   v_action         errors.action%TYPE;
   v_business_date  errors.business_date%TYPE := TRUNC(SYSDATE)-1;
   nt_customer_data bulk_load_ntt := bulk_load_ntt();

BEGIN

   v_action := 'Fetch source data';
   SELECT customer_ot(
             object_id,   --<-- generic_id in lieu of customer_id
             object_type, --<-- first_name
             object_name, --<-- last_name
             created      --<-- start_date
             )
   BULK COLLECT INTO nt_customer_data
   FROM   all_objects
   WHERE  ROWNUM < 10;

   /*
   || Add two duplicates to ensure we hit SAVE EXCEPTIONS...
   */
   FOR i IN 1 .. 2 LOOP
      nt_customer_data.EXTEND;
      nt_customer_data(nt_customer_data.LAST) := nt_customer_data(i); 
   END LOOP;

   v_action := 'Customer load';
   FORALL i IN 1 .. nt_customer_data.COUNT SAVE EXCEPTIONS
      INSERT INTO (SELECT customer_ot( customer_id,
                                       first_name,
                                       last_name,
                                       start_date )
                   FROM   customers)
      VALUES (TREAT(nt_customer_data(i) AS customer_ot));

EXCEPTION
   WHEN error.bulk_exceptions THEN
      error.log( p_owner         => USER,
                 p_package       => v_package,
                 p_procedure     => v_procedure,
                 p_action        => v_action,
                 p_business_date => v_business_date,
                 p_business_data => nt_customer_data );

      ROLLBACK; --<-- error package should encapsulate this...
      RAISE;    --<-- error package should also encapsulate this...
END;
/

exec print_table('SELECT * FROM errors');

