
set echo on

CREATE OR REPLACE PACKAGE error AS

   PROCEDURE log( p_owner         IN errors.package_owner%TYPE,
                  p_package       IN errors.package_name%TYPE,
                  p_procedure     IN errors.procedure_name%TYPE,
                  p_action        IN errors.action%TYPE,
                  p_business_date IN errors.business_date%TYPE,
                  p_business_key  IN errors.business_key%TYPE DEFAULT NULL,
                  p_error         IN VARCHAR2 DEFAULT NULL );

   bulk_exceptions EXCEPTION;
   PRAGMA EXCEPTION_INIT(bulk_exceptions, -24381);

   PROCEDURE log( p_owner         IN errors.package_owner%TYPE,
                  p_package       IN errors.package_name%TYPE,
                  p_procedure     IN errors.procedure_name%TYPE,
                  p_action        IN errors.action%TYPE,
                  p_business_date IN errors.business_date%TYPE,
                  p_business_data IN bulk_load_ntt );

END;
/

CREATE OR REPLACE PACKAGE BODY error AS

   PROCEDURE log( p_owner         IN errors.package_owner%TYPE,
                  p_package       IN errors.package_name%TYPE,
                  p_procedure     IN errors.procedure_name%TYPE,
                  p_action        IN errors.action%TYPE,
                  p_business_date IN errors.business_date%TYPE,
                  p_business_key  IN errors.business_key%TYPE DEFAULT NULL,
                  p_error         IN VARCHAR2 DEFAULT NULL ) IS

      v_error errors.error_msg%TYPE := NVL(p_error,SQLERRM);

      PRAGMA AUTONOMOUS_TRANSACTION;

   BEGIN

      INSERT INTO errors 
         ( package_owner, package_name, procedure_name, action,
           business_date, business_key, error_ts, error_msg )
      VALUES
         ( p_owner, p_package, p_procedure, p_action,
           p_business_date, p_business_key, SYSTIMESTAMP, v_error );

      COMMIT;

   END log;

   PROCEDURE log( p_owner         IN errors.package_owner%TYPE,
                  p_package       IN errors.package_name%TYPE,
                  p_procedure     IN errors.procedure_name%TYPE,
                  p_action        IN errors.action%TYPE,
                  p_business_date IN errors.business_date%TYPE,
                  p_business_data IN bulk_load_ntt ) IS

      v_error_index PLS_INTEGER;
      v_error_code  PLS_INTEGER;

   BEGIN

      /*
      || Simply log the errors...
      */
      FOR i IN 1 .. SQL%BULK_EXCEPTIONS.COUNT LOOP

         v_error_index := SQL%BULK_EXCEPTIONS(i).error_index;
         v_error_code  := SQL%BULK_EXCEPTIONS(i).error_code;

         error.log( p_owner         => p_owner,
                    p_package       => p_package,
                    p_procedure     => p_procedure,
                    p_action        => p_action,
                    p_business_date => p_business_date,
                    p_business_key  => p_business_data(v_error_index).print_key(),
                    p_error         => SQLERRM(-1 * v_error_code) );
 
      END LOOP;

   END log;

END error;
/


